/* Application Information */
#define APP_NAME "AudConvert"
#define VERSION "0.50"
#define TITLE_BAR APP_NAME " " VERSION
#define COPYRIGHT "Copyright (C) 2005 Kris Kersey (Augustus)"
#define WEBSITE "http://www.linuxhardware.org"
#define WEBSITE_LABEL "LinuxHardware.org"
#define LICENSE1 APP_NAME " " VERSION "\n" COPYRIGHT "\n\n" \
    "This program is free software; you can redistribute it and/or\n" \
    "modify it under the terms of the GNU General Public License\n" \
    "as published by the Free Software Foundation; either version 2\n" \
    "of the License, or (at your option) any later version.\n"
#define LICENSE2 "\n" \
    "This program is distributed in the hope that it will be useful,\n" \
    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n" \
    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n" \
    "GNU General Public License for more details.\n" \
    "\n" \
    "You should have received a copy of the GNU General Public License\n" \
    "along with this program; if not, write to the Free Software\n" \
    "Foundation, Inc., 51 Franklin Street, Fifth Floor,\n" \
    "Boston, MA 02110-1301, USA.\n"

/* Maximum number of threads. */
#define MAX_THREADS 8

/* Message Queue for IPC */
#define AUDCONVERT_MQUE "/audconvert_mque"

/* Semaphore used to keep track of number of running threads. */
#define AUDCONVERT_SEM  "/audconvert_sem"

/* FIFO prefix for communicating between decoders and encoders. */
#define AUDCONVERT_FIFO "/tmp/audconvert"

/* arbitrary max length of strings */
#define MAXLENGTH 1001

/* arbitrary max number of arguements passed to external apps */
#define MAXARGS 25

/* Pretty identifiers. */
enum { SUCCESS, FAILURE };

/* Types of messages. */
enum mtypes {
    PBARUPDATE=1,
    SBARUPDATE,
    DONE
};

/* Message structure. */
struct bar_info_msg {
    int mtype;
    int bar_id;
    char bar_value[MAXLENGTH];
};

/* Identifier sent to specify whether we're opening an input or output
 * directory dialog.
 */
enum inoutid {
    INPUT=1,
    OUTPUT
};

/* Used to set flags in audconvert.c from other .c files. */
enum setables {
    FLATTEN,
    JUSTGO,     /* Not setable but getable. :-) */
    BENCHMARK,  /* Not setable but getable. :-) */
    OUTPUTBITRATE,
    OGGIN,
    MP3IN,
    FLACIN,
    LAMEOUT,
    OGGOUT,
    THREADS
};

/* Supported input formats. */
enum inputformats {
    OGG,
    MP3,
    FLAC
};

/* Data we pass around used to keep track of the file we're working on. */
struct encode_data {
    char file_name[MAXLENGTH];          /* input filename */
    char output_dir[MAXLENGTH];
    char output_filename[MAXLENGTH];
    char complete_output[MAXLENGTH];

    int thread_num;

    char title[MAXLENGTH];         /* Title ID */
    char artist[MAXLENGTH];        /* Artist ID */
    char genre[MAXLENGTH];         /* Genre ID */
    char album[MAXLENGTH];         /* Album ID */
    char tracknumber[MAXLENGTH];   /* Track Number ID */
    char date[MAXLENGTH];          /* Date ID */
    char comment[MAXLENGTH];       /* Comment ID */

    int inputformat;
};

/* Declarations for accessable functions in audconvert.c. */
void *enter_dir_start( void *ptr );
int set_input_ouput_dir( const char *inputdir, const char *outputdir );
int set_value( int field, int value );
int get_value( int field );
int set_child( int number, int value );
void kill_children();
