#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include "audconvert.h"
#include "gui.h"
#include "drivers.h"

/* non-ansi defines */
int snprintf(char *str, size_t size, const char *format, ...);

void *ogg_input_thread( void *ptr )
{
    char log_message[MAXLENGTH];

    struct encode_data *file_info = (struct encode_data *) ptr;

    char fifo0_name[MAXLENGTH];
    char fifo1_name[MAXLENGTH];

    int fifo0 = 0;
    int fifo1 = 1;

    sprintf( fifo0_name, "%s%d", AUDCONVERT_FIFO,
             file_info->thread_num * 2 );
    sprintf( fifo1_name, "%s%d", AUDCONVERT_FIFO,
             (file_info->thread_num * 2)+ 1);

    fifo0 = open( fifo0_name, O_RDONLY );
    if ( fifo0 == -1 )
    {
        perror( "open of FIFO0 failed" );
    }

    fifo1 = open( fifo1_name, O_WRONLY );
    if ( fifo1 == -1 )
    {
        perror( "open of AUDCONVERT_FIFO1 failed" );
    }

    if ( fifo0 != STDIN_FILENO )
    {
        if ( dup2( fifo0, STDIN_FILENO ) != STDIN_FILENO )
        {
            snprintf( log_message, MAXLENGTH,
                      "dup2 error to stdin!\n" );
            print_log( log_message );
        }

        close( fifo0 );
    }

    if ( fifo1 != STDOUT_FILENO )
    {
        if ( dup2( fifo1, STDOUT_FILENO ) != STDOUT_FILENO )
        {
            snprintf( log_message, MAXLENGTH,
                      "dup2 error to stdout!\n" );
            print_log( log_message );
        }

        close( fifo1 );
    }

    /* if ( execlp( "ogg123", "ogg123", "-q", "-d", "wav", "-f", "-", */
    if ( execlp( "oggdec", "oggdec", "-Q", "-o", "-",
                 "-", NULL ) < 0 )
    {
        snprintf( log_message, MAXLENGTH,
                  "Exec of oggdec failed: %s\n", strerror( errno ) );
        print_log( log_message );
    }

    return NULL;
}

void *mpg123_input_thread( void *ptr )
{
    char log_message[MAXLENGTH];

    struct encode_data *file_info = (struct encode_data *) ptr;

    char fifo0_name[MAXLENGTH];
    char fifo1_name[MAXLENGTH];

    int fifo0 = 0;
    int fifo1 = 1;

    sprintf( fifo0_name, "%s%d", AUDCONVERT_FIFO,
             file_info->thread_num * 2 );
    sprintf( fifo1_name, "%s%d", AUDCONVERT_FIFO,
             (file_info->thread_num * 2)+ 1);

    fifo0 = open( fifo0_name, O_RDONLY );
    if ( fifo0 == -1 )
    {
        perror( "open of FIFO0 failed" );
    }

    fifo1 = open( fifo1_name, O_WRONLY );
    if ( fifo1 == -1 )
    {
        perror( "open of AUDCONVERT_FIFO1 failed" );
    }

    if ( fifo0 != STDIN_FILENO )
    {
        if ( dup2( fifo0, STDIN_FILENO ) != STDIN_FILENO )
        {
            snprintf( log_message, MAXLENGTH,
                      "dup2 error to stdin!\n" );
            print_log( log_message );
        }

        close( fifo0 );
    }

    if ( fifo1 != STDOUT_FILENO )
    {
        if ( dup2( fifo1, STDOUT_FILENO ) != STDOUT_FILENO )
        {
            snprintf( log_message, MAXLENGTH,
                      "dup2 error to stdout!\n" );
            print_log( log_message );
        }

        close( fifo1 );
    }

    if ( execlp( "mpg123", "mpg123", "-q", "-w", "-", "-", NULL ) < 0 )
    {
        snprintf( log_message, MAXLENGTH,
                  "Exec of mpg123 failed: %s\n", strerror( errno ) );
        print_log( log_message );
    }

    return NULL;
}

void *flac_input_thread( void *ptr )
{
    char log_message[MAXLENGTH];

    struct encode_data *file_info = (struct encode_data *) ptr;

    char fifo0_name[MAXLENGTH];
    char fifo1_name[MAXLENGTH];

    int fifo0 = 0;
    int fifo1 = 1;

    sprintf( fifo0_name, "%s%d", AUDCONVERT_FIFO,
             file_info->thread_num * 2 );
    sprintf( fifo1_name, "%s%d", AUDCONVERT_FIFO,
             (file_info->thread_num * 2)+ 1);

    fifo0 = open( fifo0_name, O_RDONLY );
    if ( fifo0 == -1 )
    {
        perror( "open of FIFO0 failed" );
    }

    fifo1 = open( fifo1_name, O_WRONLY );
    if ( fifo1 == -1 )
    {
        perror( "open of FIFO1 failed" );
    }

    if ( fifo0 != STDIN_FILENO )
    {
        if ( dup2( fifo0, STDIN_FILENO ) != STDIN_FILENO )
        {
            snprintf( log_message, MAXLENGTH,
                      "dup2 error to stdin!\n" );
            print_log( log_message );
        }

        close( fifo0 );
    }

    if ( fifo1 != STDOUT_FILENO )
    {
        if ( dup2( fifo1, STDOUT_FILENO ) != STDOUT_FILENO )
        {
            snprintf( log_message, MAXLENGTH,
                      "dup2 error to stdout!\n" );
            print_log( log_message );
        }

        close( fifo1 );
    }

    if ( execlp( "flac", "flac", "-d", "-c", "-s",
                 "-", NULL ) < 0 )
    {
        snprintf( log_message, MAXLENGTH,
                  "Exec of flac failed: %s\n", strerror( errno ) );
        print_log( log_message );
    }

    return NULL;
}


void *lame_output_thread( void *ptr )
{
    char log_message[MAXLENGTH];

    struct encode_data *file_info = (struct encode_data *) ptr;

    char fifo1_name[MAXLENGTH];

    int input = 0;

    char *lameargs[MAXARGS];   /* Arguements to pass to lame. */
    int i = 0;

    sprintf( fifo1_name, "%s%d", AUDCONVERT_FIFO,
             (file_info->thread_num * 2) + 1);

    input = open( fifo1_name, O_RDONLY );
    if ( input == -1 )
    {
        perror( "open of AUDCONVERT_FIFO1 failed" );
    }

    if ( input != STDIN_FILENO )
    {
        if ( dup2( input, STDIN_FILENO ) != STDIN_FILENO )
        {
            snprintf( log_message, MAXLENGTH,
                      "dup2 error to stdin!\n" );
            print_log( log_message );
        }

        close( input );
    }

    /* Setup the lame arguements. */
    for ( i = 0; i < MAXARGS; i++ )
    {
        if ( ( lameargs[i] = malloc( MAXLENGTH * sizeof(char) ) ) == NULL )
        {
            snprintf( log_message, MAXLENGTH,
                      "malloc failed!\n" );
            print_log( log_message );
        }
    }
    strncpy( lameargs[0], "lame", MAXLENGTH );
    strncpy( lameargs[1], "--quiet", MAXLENGTH );
    strncpy( lameargs[2], "-m", MAXLENGTH );
    strncpy( lameargs[3], "j", MAXLENGTH );
    strncpy( lameargs[4], "-q", MAXLENGTH );
    strncpy( lameargs[5], "2", MAXLENGTH );
    strncpy( lameargs[6], "-b", MAXLENGTH );
    snprintf( lameargs[7], MAXLENGTH, "%d",
              get_value( OUTPUTBITRATE ) );
    strncpy( lameargs[8], "--tt", MAXLENGTH );
    strncpy( lameargs[9], file_info->title, MAXLENGTH );
    strncpy( lameargs[10], "--ta", MAXLENGTH );
    strncpy( lameargs[11], file_info->artist, MAXLENGTH );
    strncpy( lameargs[12], "--tl", MAXLENGTH );
    strncpy( lameargs[13], file_info->album, MAXLENGTH );
    strncpy( lameargs[14], "--tn", MAXLENGTH );
    strncpy( lameargs[15], file_info->tracknumber, MAXLENGTH );
    strncpy( lameargs[16], "--tg", MAXLENGTH );
    strncpy( lameargs[17], file_info->genre, MAXLENGTH );
    strncpy( lameargs[18], "--ty", MAXLENGTH );
    strncpy( lameargs[19], file_info->date, MAXLENGTH );
    strncpy( lameargs[20], "--tc", MAXLENGTH );
    strncpy( lameargs[21], file_info->comment, MAXLENGTH );
    strncpy( lameargs[22], "-", MAXLENGTH );
    strncpy( lameargs[23], file_info->complete_output, MAXLENGTH );
    free( lameargs[24] );
    lameargs[24] = NULL;

#ifdef DEBUG
    fprintf( stderr, "LAME ARGS:\n" );
    for ( i = 0; i < (MAXARGS - 1); i++ )
    {
        printf( "argv[%d]: %s\n", i, lameargs[ i ] );
    }
#endif

    /* Exec lame. */
    if ( execvp( "lame", lameargs ) < 0 )
    {
        snprintf( log_message, MAXLENGTH,
                  "Exec of lame failed: %s\n", strerror( errno ) );
        print_log( log_message );
    }

    /* Cleanup memory. */
    for ( i = 0; i < (MAXARGS - 1); i++ )
    {
        free( lameargs[ i ] );
    }

    return NULL;
}

void *oggenc_output_thread( void *ptr )
{
    char log_message[MAXLENGTH];

    struct encode_data *file_info = (struct encode_data *) ptr;

    char fifo1_name[MAXLENGTH];

    int input = 0;

    char *oggencargs[MAXARGS];   /* Arguements to pass to oggenc. */
    int i = 0;

    sprintf( fifo1_name, "%s%d", AUDCONVERT_FIFO,
             (file_info->thread_num * 2) + 1);

    input = open( fifo1_name, O_RDONLY );
    if ( input == -1 )
    {
        perror( "open of AUDCONVERT_FIFO failed" );
    }

    if ( input != STDIN_FILENO )
    {
        if ( dup2( input, STDIN_FILENO ) != STDIN_FILENO )
        {
            snprintf( log_message, MAXLENGTH,
                      "dup2 error to stdin!\n" );
            print_log( log_message );
        }

        close( input );
    }

    /* Setup the lame arguements. */
    for ( i = 0; i < MAXARGS; i++ )
    {
        if ( ( oggencargs[i] = malloc( MAXLENGTH * sizeof(char) ) ) == NULL )
        {
            snprintf( log_message, MAXLENGTH,
                      "malloc failed!\n" );
            print_log( log_message );
        }
    }
    strncpy( oggencargs[0], "oggenc", MAXLENGTH );
    strncpy( oggencargs[1], "-Q", MAXLENGTH );
    strncpy( oggencargs[2], "-q", MAXLENGTH );
    strncpy( oggencargs[3], "3", MAXLENGTH );
    strncpy( oggencargs[4], "-b", MAXLENGTH );
    snprintf( oggencargs[5], MAXLENGTH, "%d",
              get_value( OUTPUTBITRATE ) );
    strncpy( oggencargs[6], "-t", MAXLENGTH );
    strncpy( oggencargs[7], file_info->title, MAXLENGTH );
    strncpy( oggencargs[8], "-a", MAXLENGTH );
    strncpy( oggencargs[9], file_info->artist, MAXLENGTH );
    strncpy( oggencargs[10], "-l", MAXLENGTH );
    strncpy( oggencargs[11], file_info->album, MAXLENGTH );
    strncpy( oggencargs[12], "-N", MAXLENGTH );
    strncpy( oggencargs[13], file_info->tracknumber, MAXLENGTH );
    strncpy( oggencargs[14], "-G", MAXLENGTH );
    strncpy( oggencargs[15], file_info->genre, MAXLENGTH );
    strncpy( oggencargs[16], "-d", MAXLENGTH );
    strncpy( oggencargs[17], file_info->date, MAXLENGTH );
    strncpy( oggencargs[18], "-c", MAXLENGTH );
    snprintf( oggencargs[19], MAXLENGTH, "comment=%s",
              file_info->comment );
    strncpy( oggencargs[20], "-o", MAXLENGTH );
    strncpy( oggencargs[21], file_info->complete_output, MAXLENGTH );
    strncpy( oggencargs[22], "-", MAXLENGTH );
    free( oggencargs[23] );
    oggencargs[23] = NULL;

#ifdef DEBUG
    fprintf( stderr, "OGGENC ARGS:\n" );
    for ( i = 0; i < (MAXARGS - 1); i++ )
    {
        printf( "argv[%d]: %s\n", i, oggencargs[ i ] );
    }
#endif

    /* Exec oggenc. */
    if ( execvp( "oggenc", oggencargs ) < 0 )
    {
        snprintf( log_message, MAXLENGTH,
                  "Exec of oggenc failed: %s\n", strerror( errno ) );
        print_log( log_message );
    }

    /* Cleanup memory. */
    for ( i = 0; i < (MAXARGS - 1); i++ )
    {
        free( oggencargs[ i ] );
    }

    return NULL;
}

